@@ -171,6 +171,39 @@
   }
 }
 
+.get_prior_factor_levels      <- function(prior){
+  if(is.prior.independent(prior)){
+    return(attr(prior, "levels"))
+  }else if(is.prior.treatment(prior)){
+    return(attr(prior, "levels") - 1)
+  }else if(is.prior.orthonormal(prior)){
+    return(attr(prior, "levels") - 1)
+  }else if(is.prior.meandif(prior)){
+    return(attr(prior, "levels") - 1)
+  }else if(is.prior.point(prior)){
+    # allow to deal with spike priors assuming there is an intercept & the prior is not independent
+    return(attr(prior, "levels") - 1)
+  }
+}
+.get_prior_factor_level_names <- function(prior){
+  if(is.null(attr(prior, "level_names"))){
+    if(is.prior.independent(prior)){
+      return(1:.get_prior_factor_levels(prior))
+    }else{
+      return(1:(.get_prior_factor_levels(prior)+1))
+    }
+  }else{
+    return(attr(prior, "level_names"))
+  }
+}
+.is_prior_interaction         <- function(prior){
+  if(is.null(attr(prior, "interaction"))){
+    return(FALSE)
+  }else{
+    return(attr(prior, "interaction"))
+  }
+}
+
 #' @title Reports whether x is a a prior object
 #'
 #' @description Reports whether x is a a prior object. Note that
@@ -207,7 +240,9 @@
 #' @export is.prior.weightfunction
 #' @export is.prior.factor
 #' @export is.prior.orthonormal
-#' @export is.prior.dummy
+#' @export is.prior.meandif
+#' @export is.prior.treatment
+#' @export is.prior.independent
 #' @export is.prior.spike_and_slab
 #' @name is.prior
 NULL
@@ -257,14 +292,21 @@ is.prior.orthonormal     <- function(x){
   inherits(x, "prior.orthonormal")
 }
 #' @rdname is.prior
-is.prior.dummy           <- function(x){
-  inherits(x, "prior.dummy")
+is.prior.treatment       <- function(x){
+  inherits(x, "prior.treatment")
+}
+#' @rdname is.prior
+is.prior.independent     <- function(x){
+  inherits(x, "prior.independent")
 }
 #' @rdname is.prior
 is.prior.spike_and_slab  <- function(x){
   inherits(x, "prior.spike_and_slab")
 }
-
+#' @rdname is.prior
+is.prior.meandif         <- function(x){
+  inherits(x, "prior.meandif")
+}
 
 .check_prior <- function(prior, name = "prior"){
   if(!is.prior(prior))
